/**
 * TalkingQuery.java
 * Created on 2005/05/20
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi.node;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * m[hƂ̒ʐMNG
 * @author es
 */
public class TalkingQuery {

    protected static final String ipPtn = "(\\d{1,3})\\.(\\d{1,3})\\.(\\d{1,3})\\.(\\d{1,3})";

    /**
     * mIȉŐI
     */
    public static final int RESULT_ACK = 0;

    /**
     * ُI
     */
    public static final int RESULT_ERR = 2;

    /**
    * ےIȉŐI
     */
    public static final int RESULT_NEG = 1;

    //urlpregex
    protected static final Pattern urlPtn = Pattern.compile("\\/(\\w+)(\\/(.+)?)?");

    private String argument;
    private final Charset charset = Charset.forName("UTF-8");

    private String command;
    private String[] response;
    private URL server;
    private URL target;
    private String ua;

    /**
     * RXgN^.<br>
     * [U[G[WFg̓ftHg
     * @param query http://(node:port)/(command)[/argument...] `̊SCURL
     */
    public TalkingQuery(String servantUrl, String argument) {
        this(servantUrl, argument, "shinGETsu/0.5");
    }

    /**
     * @param servantUrl
     * @param argument
     * @param userAgent
     */
    public TalkingQuery(String servantUrl, String argument, String userAgent) {
        try {
            this.target = new URL(servantUrl + argument);
            this.server = new URL(servantUrl);
        } catch (MalformedURLException e) {
            throw new IllegalArgumentException(e.getMessage());
        }
        this.ua = userAgent;
        parseQuery(argument);
    }

    /**
     * 擾 (ȂȂnull)
     * @return
     */
    public String getArgument() {
        return argument;
    }

    /**
     * R}h擾
     * @return
     */
    public String getCommand() {
        return command;
    }

    /*
     * QƗp
     * @return
     */
    String getFile() {
        return target.getFile();
    }

    /**
     * T[o[擾
     * @return
     */
    public String[] getResponse() {
        return response;
    }

    /**
     * ʐMm[hURL擾
     * @return
     */
    public URL getServantURL() {
        return server;
    }

    /**
     * URLR}hƈ擾
     */
    protected void parseQuery(String str) {
        Matcher m = urlPtn.matcher(str);
        if(!m.matches()) {
            throw new IllegalArgumentException("");
        }
        command = (m.group(1) == null) ? "/" : m.group(1);
        argument = m.group(3);
    }

    /**
     * T[o[ʐMʂ
     * @return ʐM
     */
    protected int parseResponse() throws ProtocolException {
        int result = RESULT_ACK;
        try {
            if (command.equals("ping")) {
                result =
                    (response[0].trim().equals("PONG") && response[1].trim().matches(ipPtn)) ? RESULT_ACK : RESULT_ERR;
            } else if (command.equals("node")) {
                result = response.length > 0 ? RESULT_ACK : RESULT_NEG;
            } else if (command.equals("join")) {
                result = response[0].trim().equals("WELCOME") ? RESULT_ACK : RESULT_ERR;
            } else if (command.equals("bye")) {
                result = response[0].trim().equals("BYEBYE") ? RESULT_ACK : RESULT_ERR;
            } else if (command.equals("have")) {
                String temp = response[0].trim();
                if (temp.equals("YES")) {
                    result = RESULT_ACK;
                } else if (temp.equals("NO")) {
                    result = RESULT_NEG;
                } else {
                    result = RESULT_ERR;
                }
            }
        } catch (Exception e) {
            result = RESULT_ERR;
        }
        return result;
    }

    /**
     * T[o[ݒ
     * @param strings
     */
    protected void setResponse(String[] strings) {
        response = strings;
    }

    /**
     * ʐMs
     * @return ʐM
     * @throws IOException 炩̗O
     */
    public int talk() throws IOException {
        HttpURLConnection conn = (HttpURLConnection) target.openConnection();
        conn.setRequestProperty("User-Agent", ua);
        conn.setRequestProperty("Accept", "text/plain");
        conn.setRequestProperty("Accept-charset", charset.name());
        conn.connect();

        List lines = new ArrayList();
        BufferedReader reader = new BufferedReader(new InputStreamReader(conn.getInputStream(), charset));
        for (String l = reader.readLine(); l != null; l = reader.readLine()) {
            lines.add(l);
        }
        reader.close();
        conn.disconnect();

        setResponse((String[]) lines.toArray(new String[lines.size()]));
        return parseResponse();
    }

    /* ( Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return target.toString();
    }

}
